/**
 * Copyright (C) 2009 STMicroelectronics
 *
 * This file is part of "Mind Compiler" is free software: you can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: mind@ow2.org
 *
 * Authors: Matthieu Leclercq
 * Contributors: 
 */

package org.ow2.mind.idl;

import static org.objectweb.fractal.adl.NodeUtil.cloneGraph;

import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.ow2.mind.idl.IDLLoader.AbstractDelegatingIDLLoader;
import org.ow2.mind.idl.ast.IDL;
import org.ow2.mind.idl.ast.IDLASTHelper;
import org.ow2.mind.idl.ast.InterfaceDefinition;
import org.ow2.mind.idl.ast.Method;

import com.google.inject.Inject;

public class ExtendsInterfaceLoader extends AbstractDelegatingIDLLoader {

  @Inject
  protected InterfaceReferenceResolver interfaceReferenceResolverItf;

  // ---------------------------------------------------------------------------
  // Implementation of the IDLLoader interface
  // ---------------------------------------------------------------------------

  public IDL load(final String name, final Map<Object, Object> context)
      throws ADLException {
    final IDL idl = clientIDLLoaderItf.load(name, context);

    if (idl instanceof InterfaceDefinition) {
      final InterfaceDefinition itf = (InterfaceDefinition) idl;
      if (itf.getExtends() != null) {
        final InterfaceDefinition extendedItf = interfaceReferenceResolverItf
            .resolve(itf.getExtends(), itf, context);

        if (!IDLASTHelper.isUnresolvedInterfaceDefinitionNode(extendedItf)) {
          mergeInterface(itf, extendedItf);
          ExtendedInterfaceDecorationHelper.addExtendedInterface(itf,
              extendedItf);
        }
      }
    }

    return idl;
  }

  // ---------------------------------------------------------------------------
  // Utility method
  // ---------------------------------------------------------------------------

  protected void mergeInterface(final InterfaceDefinition itf,
      final InterfaceDefinition superItf) {
    // remove method from itf
    final Method[] meths = itf.getMethods();
    for (final Method method : meths) {
      itf.removeMethod(method);
    }
    // add clone of inherited methods
    for (final Method method : superItf.getMethods()) {
      itf.addMethod(cloneGraph(method));
    }
    // re-add methods from itf. (this ensure that methods defined in "itf" are
    // after methods defined in "superItf"
    for (final Method method : meths) {
      itf.addMethod(method);
    }
  }
}
